﻿// CDC_vKBMSTest.cpp : This file contains the 'main' function. Program execution begins and ends there.
//

#include <windows.h>
#include <SetupAPI.h>
#include <tchar.h>
#include <iostream>
#include <cstring>
#include <atlstr.h>

#pragma comment(lib, "Setupapi.lib")

#define MY_USB_PID_VID	_T("VID_1209&PID_C55C")

class ComPortException : public std::exception {
public:
	ComPortException(DWORD errorCode) : errorCode(errorCode) {}

	DWORD getErrorCode() const {
		return errorCode;
	}

private:
	DWORD errorCode;
};

// 对串口portName 发送数据
// 无法打开串口返回 1
// 无法设置串口参数 2
void SendToComPort(const int port, const char* data) {
	int Result = 0;
	HANDLE hCom = INVALID_HANDLE_VALUE;
	TCHAR portName[10]; // 用于存储 "COMp" 字符串

// 将整数 p 转换为 "COMp" 字符串
#ifdef UNICODE
	swprintf(portName, 10, _T("\\\\.\\COM%d"), port);
#else
	sprintf(portName, "COM%d", port);
#endif
	try {
		// 打开串口
		hCom = CreateFile(portName,
			GENERIC_READ | GENERIC_WRITE,
			0,
			NULL,
			OPEN_EXISTING,
			0,
			NULL);

		if (hCom == INVALID_HANDLE_VALUE) {
			throw ComPortException(GetLastError());
		}

		// 设置串口参数
		DCB dcb;
		SecureZeroMemory(&dcb, sizeof(DCB));
		dcb.DCBlength = sizeof(DCB);

		if (!GetCommState(hCom, &dcb)) {
			throw ComPortException(GetLastError());
		}

		dcb.BaudRate = CBR_115200;  // 波特率
		dcb.ByteSize = 8;         // 数据位
		dcb.StopBits = ONESTOPBIT; // 停止位
		dcb.Parity = NOPARITY;    // 校验位

		if (!SetCommState(hCom, &dcb)) {
			throw ComPortException(GetLastError());
		}

		// 设置超时参数
		COMMTIMEOUTS timeouts;
		timeouts.ReadIntervalTimeout = 50;
		timeouts.ReadTotalTimeoutConstant = 50;
		timeouts.ReadTotalTimeoutMultiplier = 10;
		timeouts.WriteTotalTimeoutConstant = 50;
		timeouts.WriteTotalTimeoutMultiplier = 10;

		if (!SetCommTimeouts(hCom, &timeouts)) {
			throw ComPortException(GetLastError());
		}

		// 发送数据
		DWORD bytesWritten;
		if (!WriteFile(hCom, data, 10, &bytesWritten, NULL)) {
			std::cerr << "Failed to write to COM port." << std::endl;
		}
		else {
			std::cout << "Successfully sent data to COM port: [" << port << "]" << std::endl;
		}
		// 关闭串口
		if (hCom != INVALID_HANDLE_VALUE) {
			CloseHandle(hCom);
		}
	}
	catch (const ComPortException& ex) {
		std::cerr << "Error: " << ex.getErrorCode() << std::endl;
		if (hCom != INVALID_HANDLE_VALUE) {
			CloseHandle(hCom);
		}
	}

}

/************************************************************************/
/* 根据USB描述信息字符串中读取
/************************************************************************/
int MTGetPortFromVidPid(CString strVidPid)
{
	// 获取当前系统所有使用的设备
	int					nPort = -1;
	int					nStart = -1;
	int					nEnd = -1;
	int					i = 0;
	CString				strTemp, strName;
	DWORD				dwFlag = (DIGCF_ALLCLASSES | DIGCF_PRESENT);
	HDEVINFO			hDevInfo = INVALID_HANDLE_VALUE;
	SP_DEVINFO_DATA		sDevInfoData;
	TCHAR				szDis[2048] = { 0x00 };// 存储设备实例ID
	TCHAR				szFN[MAX_PATH] = { 0x00 };// 存储设备实例属性
	DWORD				nSize = 0;

	// 准备遍历所有设备查找USB
	hDevInfo = SetupDiGetClassDevs(NULL, L"USB", NULL, dwFlag);
	if (INVALID_HANDLE_VALUE == hDevInfo)
		goto STEP_END;

	// 开始遍历所有设备
	memset(&sDevInfoData, 0x00, sizeof(SP_DEVICE_INTERFACE_DATA));
	sDevInfoData.cbSize = sizeof(SP_DEVINFO_DATA);
	for (i = 0; SetupDiEnumDeviceInfo(hDevInfo, i, &sDevInfoData); i++)
	{
		nSize = 0;

		// 无效设备
		if (!SetupDiGetDeviceInstanceId(hDevInfo, &sDevInfoData, szDis, sizeof(szDis), &nSize))
			goto STEP_END;

		// 根据设备信息寻找VID PID一致的设备
		strTemp.Format(_T("%s"), szDis);
		strTemp.MakeUpper();
		if (strTemp.Find(strVidPid, 0) == -1)
			continue;

		// 查找设备属性
		nSize = 0;
		SetupDiGetDeviceRegistryProperty(hDevInfo, &sDevInfoData,
			SPDRP_FRIENDLYNAME,
			0, (PBYTE)szFN,
			sizeof(szFN),
			&nSize);

		// "XXX Virtual Com Port (COM7)"
		strName.Format(_T("%s"), szFN);
		if (strName.IsEmpty())
			//goto STEP_END;
			continue;

		// 寻找串口信息
		nStart = strName.Find(_T("(COM"), 0);
		nEnd = strName.Find(_T(")"), 0);
		if (nStart == -1 || nEnd == -1)
			//goto STEP_END;
			continue;

		strTemp = strName.Mid(nStart + 4, nEnd - nStart - 2);
		nPort = _ttoi(strTemp);

	}
STEP_END:

	// 关闭设备信息集句柄
	if (hDevInfo != INVALID_HANDLE_VALUE)
	{
		SetupDiDestroyDeviceInfoList(hDevInfo);
		hDevInfo = INVALID_HANDLE_VALUE;
	}

	return nPort;
}

int main()
{
	int Port;
	char Data[10];
	printf("Virutal KB MS Test\n");
	Port = MTGetPortFromVidPid(MY_USB_PID_VID);
	if (Port == -1) {
		printf("No device is found\n");
		goto EndProgram;
	}
	else {
		printf("Found COM%d\n",Port);
	}
	Sleep(5000);

	// 测试鼠标移动
	memset(Data,0,sizeof(Data));
	Data[0] = 2; // 鼠标
	Data[2] = 50;
	SendToComPort(Port,(char *)&Data);
	Sleep(300);
	Data[2] = 0; Data[3] = 50;
	SendToComPort(Port, (char*)&Data);
	Sleep(300);
	Data[2] = -50; Data[3] = 0x00;
	SendToComPort(Port, (char*)&Data);
	Sleep(300);
	Data[2] = 0; Data[3] = -50;
	SendToComPort(Port, (char*)&Data);
	Sleep(300);

	// 测试键盘数据
	memset(Data, 0, sizeof(Data));
	Data[0] = 1; // 键盘
	// 发送GUI信息
	Data[1] = 0x08;
	SendToComPort(Port, (char*)&Data);
	Sleep(1000);
	memset(Data, 0, sizeof(Data));
	Data[0] = 1; // 键盘
	SendToComPort(Port, (char*)&Data);
	Sleep(300);

	// 测试键盘数据
	memset(Data, 0, sizeof(Data));
	Data[0] = 1; // 键盘
	// 发送按键信息
	Data[2] = 0x04; 
	Data[3] = 0x0F; //'l'
	Data[4] = 0x04; //'a ' 
	Data[5] = 0x05; //'b'
	Data[6] = 0x1d; //'z'
	SendToComPort(Port, (char*)&Data);
	Sleep(300);
	memset(Data, 0, sizeof(Data));
	Data[0] = 1; // 键盘
	// 抬起按键
	SendToComPort(Port, (char*)&Data);
	Sleep(300);



	// 测试LED
	memset(Data, 0, sizeof(Data));
	Data[0] = 3; // LED
	Data[1] = 0xFF;
	SendToComPort(Port, (char*)&Data);
	Sleep(500);
	memset(Data, 0, sizeof(Data));
	Data[0] = 3; // LED
	Data[2] = 0xFF;
	SendToComPort(Port, (char*)&Data);
	Sleep(500);
	memset(Data, 0, sizeof(Data));
	Data[0] = 3; // LED
	Data[3] = 0xFF;
	SendToComPort(Port, (char*)&Data);
	Sleep(500);
	memset(Data, 0, sizeof(Data));
	Data[0] = 3; // LED
	SendToComPort(Port, (char*)&Data);
	Sleep(500);

EndProgram:
	printf("End\n");
}
